/*	
 *	clipz80.c
 *
 *	P/ECE Z80 Emulator
 *
 *	CLiP - Common Library for P/ECE
 *	Copyright (C) 2001-2003 Naoyuki Sawa
 *
 *	* Sat Nov 22 06:00:00 JST 2003 Naoyuki Sawa
 *	- 쐬JnB
 */
#include "app.h"
#include "clipz8op.h"

/****************************************************************************
 *
 ****************************************************************************/

/*
 *	IyR[h}bv
 */
const Z80OP z80_op_table[256/*XX*/] = {
#include "clipz8t1.h" /* [z80.xls]XX */
};
const Z80OP z80_op_table_CB[256/*XX*/] = {
#include "clipz8t2.h" /* [z80.xls]CB */
};
const Z80OP z80_op_table_ED[256/*XX*/] = {
#include "clipz8t3.h" /* [z80.xls]ED */
};
const Z80OP z80_op_table_DD[256/*XX*/] = {
#include "clipz8t4.h" /* [z80.xls]DD */
};
const Z80OP z80_op_table_FD[256/*XX*/] = {
#include "clipz8t5.h" /* [z80.xls]FD */
};
const Z80OP z80_op_table_DDCB[256/*XX*/] = {
#include "clipz8t6.h" /* [z80.xls]DDCB */
};
const Z80OP z80_op_table_FDCB[256/*XX*/] = {
#include "clipz8t7.h" /* [z80.xls]FDCB */
};

/*
 *	j[jbNe[u
 */
#undef Z80FN_
#define Z80FN_(OP, FN)	OP "\0"
const char z80_op_name_table[] = {
	"\0" /* ItZbg0̓k */
#include "clipz8ls.h" /* [z80.xls]List!C10 */
};
#undef Z80FN_

/****************************************************************************
 *
 ****************************************************************************/

/* Z80RAZbg܂B
 * [in]
 *	z80		Z80\́B
 *	read		ǂݍ݊O֐B
 *	write		݊O֐B
 *	in		I/Oǂݍ݊O֐B
 *	out		I/O݊O֐B
 */
void
z80_reset(Z80* z80, Z80READPROC* read, Z80WRITEPROC* write, Z80INPROC* in, Z80OUTPROC* out)
{
	ASSERT(read);
	ASSERT(write);
	ASSERT(in);
	ASSERT(out);

	memset(z80, 0, sizeof(Z80));
	z80->read = read;
	z80->write = write;
	z80->in = in;
	z80->out = out;
}

/* }XN\荞݃NGXg𔭍s܂B
 * [in]
 *	z80		Z80\́B
 *	no		荞݃[hɂĈӖقȂ܂B
 *			[h0: RST XXH ߂̃IyR[hw肵܂B
 *			[h1: Ӗ܂B
 *			[h2: 荞݃xN^̉ʃoCgw肵܂B
 * [note]
 *	* 荞݂֎~ĂA荞݃NGXg͖܂B
 *	  荞݃NGXg͕ۗ܂B
 */
void
z80_int(Z80* z80, unsigned char no)
{
	if(!IFF1) return;
	switch(IM) {
	case 0: /* IM 0 */
		DIE(); /* TODO:Ή */
	case 2: /* IM 1 (v!!1͌Ԃł) */
		IFF1 = IFF2 = 0;
		PUSH(PC);
		PC = 0x38;
		break;
	case 3: /* IM 2 (v!!1͌Ԃł) */
		DIE(); /* TODO:Ή */
	default:
		DIE();
	}
	HALT = 0;
}

/* }XNs荞݃NGXg𔭍s܂B
 * [in]
 *	z80		Z80\́B
 */
void
z80_nmi(Z80* z80)
{
	IFF2 = IFF1;
	IFF1 = 0;
	PUSH(PC);
	PC = 0x66;
	HALT = 0;
}

/* WX^_vs܂B
 * [in]
 *	z80		Z80\́B
 */
void
z80_dump(Z80* z80)
{
#define INDENT	40
	TRACE("%*s", INDENT, "");
	TRACE("F :%c%c%c%c%c%c%c%c A :%02x BC :%04x DE :%04x HL :%04x IX:%04x IY:%04x SP:%04x PC:%04x\n",
		LO(AF) & (1 << 7) ? 'S' : '-',
		LO(AF) & (1 << 6) ? 'Z' : '-',
		LO(AF) & (1 << 5) ? 'X' : '-',
		LO(AF) & (1 << 4) ? 'H' : '-',
		LO(AF) & (1 << 3) ? 'X' : '-',
		LO(AF) & (1 << 2) ? 'V' : '-',
		LO(AF) & (1 << 1) ? 'N' : '-',
		LO(AF) & (1 << 0) ? 'C' : '-',
		HI(AF), BC, DE, HL, IX, IY, SP, PC);
	TRACE("%*s", INDENT, "");
	TRACE("F':%c%c%c%c%c%c%c%c A':%02x BC':%04x DE':%04x HL':%04x R:%02x I:%02x IFF1:%02x IFF2:%02x IM:%02x\n",
		LO(AF2) & (1 << 7) ? 'S' : '-',
		LO(AF2) & (1 << 6) ? 'Z' : '-',
		LO(AF2) & (1 << 5) ? 'X' : '-',
		LO(AF2) & (1 << 4) ? 'H' : '-',
		LO(AF2) & (1 << 3) ? 'X' : '-',
		LO(AF2) & (1 << 2) ? 'V' : '-',
		LO(AF2) & (1 << 1) ? 'N' : '-',
		LO(AF2) & (1 << 0) ? 'C' : '-',
		HI(AF2), BC2, DE2, HL2, I, R, IFF1, IFF2, IM);
#undef INDENT
}

